<?php

/**
 * This file is part of ILIAS, a powerful learning management system
 * published by ILIAS open source e-Learning e.V.
 *
 * ILIAS is licensed with the GPL-3.0,
 * see https://www.gnu.org/licenses/gpl-3.0.en.html
 * You should have received a copy of said license along with the
 * source code, too.
 *
 * If this is not the case or you just want to try ILIAS, you'll find
 * us at:
 * https://www.ilias.de
 * https://github.com/ILIAS-eLearning
 *
 *********************************************************************/

declare(strict_types=1);

namespace ILIAS\Plugin\PlagScanExercise\Form;

use ilCustomInputGUI;
use ilExAssignment;
use ilFormSectionHeaderGUI;
use ILIAS\DI\Container;
use ILIAS\Plugin\PlagScanExercise\PlagScan\AssignmentConfigData;
use ILIAS\Plugin\PlagScanExercise\PlagScan\AssignmentConfigPersistence;
use ILIAS\Plugin\PlagScanExercise\PlagScan\Document;
use ILIAS\Plugin\PlagScanExercise\PlagScan\DocumentManagement;
use ilPlagScanExercisePlugin;
use ilPropertyFormGUI;
use ilSetting;

class ExOverviewAdditions
{
    protected Container $dic;
    protected ilPlagScanExercisePlugin $plugin;

    public function __construct()
    {
        global $DIC;

        $this->dic = $DIC;
        $this->plugin = ilPlagScanExercisePlugin::getInstance();
    }

    public function getForm(AssignmentConfigData $assignment): ilPropertyFormGUI
    {
        $settings = new ilSetting('psx');
        $form = new ilPropertyFormGUI();
        $section = new ilFormSectionHeaderGUI();
        $section->setTitle('PlagScan');
        $form->addItem($section);

        if ($settings->get('psx_disclosure') !== '') {
            $disclosure = new ilCustomInputGUI($this->plugin->txt('disclosure'), 'psx_disclosure');
            $disclosure->setHtml($settings->get('psx_disclosure'));
            $form->addItem($disclosure);
        }
        $status = DocumentManagement::getDocumentStatus($this->dic->user()->getId(), $assignment->getAssignmentId());
        if ($status !== 'no_submission') {
            $status_input = new ilCustomInputGUI($this->dic->language()->txt('status'), 'psx_status');
            switch ($status) {
                case 'in_progress':
                    $status_input->setHtml($this->plugin->txt('in_progress'));
                    break;
                case 'not_checked':
                    $status_input->setHtml($this->plugin->txt('not_checked'));
                    break;
                default:
                    $status_input->setHtml($this->addResultBlock($this->dic->user()->getId(), $assignment->getAssignmentId()));
            }
            $form->addItem($status_input);
        }
        return $form;
    }

    public function addResultBlock(int $user_id, int $assignment_id): string
    {
        $document = new Document();
        $document->getByUserAssignment($user_id, $assignment_id);

        $config_persistence = new AssignmentConfigPersistence($this->dic->database());
        $assignment_config = $config_persistence->getByAssignmentId($assignment_id);

        if ($assignment_config->getShareSetting() === 'never') {
            return 'Checked - PlagLevel hidden';
        }
        $assignment = new ilExAssignment($assignment_id);
        if ($assignment_config->getShareSetting() === 'deadline' && !$assignment->afterDeadline()) {
            return 'Checked - Results / PlagLevel Shown after Deadline';
        }

        $plag_level = substr($document->getPlagscanStatus(), 10);
        if ($assignment_config->getShareType() === 'percentage_link') {
            return 'PlagLevel ' . $plag_level . '% - <a href="' . $document->getShareLink() . '" target="_blank">Report</a>';
        }
        return 'PlagLevel ' . $plag_level . '%';
    }
}
