<?php

/**
 * This file is part of ILIAS, a powerful learning management system
 * published by ILIAS open source e-Learning e.V.
 *
 * ILIAS is licensed with the GPL-3.0,
 * see https://www.gnu.org/licenses/gpl-3.0.en.html
 * You should have received a copy of said license along with the
 * source code, too.
 *
 * If this is not the case or you just want to try ILIAS, you'll find
 * us at:
 * https://www.ilias.de
 * https://github.com/ILIAS-eLearning
 *
 *********************************************************************/

declare(strict_types=1);

namespace ILIAS\Plugin\PlagScanExercise\PlagScan;

use ilDBConstants;

class Document
{
    protected int $id = 0;
    protected int $assignment_exc_obj = 0;
    protected int $assignment_id = 0;
    protected int $user_id = 0;
    protected bool $user_non_consent = false;
    protected int $submission_ilias_id = 0;
    protected int $plagscan_id = 0;
    protected string $plagscan_status = '';
    protected string $share_link = '';

    public function getId(): int
    {
        return $this->id;
    }

    public function setId(int $id): void
    {
        $this->id = $id;
    }

    public function getAssignmentExcObj(): int
    {
        return $this->assignment_exc_obj;
    }

    public function setAssignmentExcObj(int $assignment_exc_obj): void
    {
        $this->assignment_exc_obj = $assignment_exc_obj;
    }

    public function getAssignmentId(): int
    {
        return $this->assignment_id;
    }

    public function setAssignmentId(int $assignment_id): void
    {
        $this->assignment_id = $assignment_id;
    }

    public function getUserId(): int
    {
        return $this->user_id;
    }

    public function setUserId(int $user_id): void
    {
        $this->user_id = $user_id;
    }

    public function getUserNonConsent(): bool
    {
        return $this->user_non_consent;
    }

    public function setUserNonConsent(bool $user_non_consent): void
    {
        $this->user_non_consent = $user_non_consent;
    }

    public function getSubmissionIliasId(): int
    {
        return $this->submission_ilias_id;
    }

    public function setSubmissionIliasId(int $submission_ilias_id): void
    {
        $this->submission_ilias_id = $submission_ilias_id;
    }

    public function getPlagscanId(): int
    {
        return $this->plagscan_id;
    }

    public function setPlagscanId(int $plagscan_id): void
    {
        $this->plagscan_id = $plagscan_id;
    }

    public function getPlagscanStatus(): string
    {
        return $this->plagscan_status;
    }

    public function setPlagscanStatus(string $plagscan_status): void
    {
        $this->plagscan_status = $plagscan_status;
    }

    public function getShareLink(): string
    {
        return $this->share_link;
    }

    public function setShareLink(string $share_link): void
    {
        $this->share_link = $share_link;
    }

    public function getByUserAssignment(int $user_id, int $assignment_id): self
    {
        global $DIC;

        $result = $DIC->database()->queryF(
            'SELECT * FROM psx_submissions WHERE assignment_id = %s AND user_id = %s',
            [ilDBConstants::T_INTEGER, ilDBConstants::T_INTEGER],
            [$assignment_id, $user_id]
        );

        if ($DIC->database()->numRows($result) > 0) {
            $row = $DIC->database()->fetchAssoc($result);

            $this->id = (int) $row['id'];
            $this->assignment_exc_obj = (int) $row['assignment_exc_obj'];
            $this->assignment_id = (int) $row['assignment_id'];
            $this->user_id = (int) $row['user_id'];
            $this->user_non_consent = $row['user_non_consent'] === '1';
            $this->submission_ilias_id = (int) $row['submission_ilias_id'];
            $this->plagscan_id = (int) $row['plagscan_id'];
            $this->plagscan_status = $row['plagscan_status'];
            $this->share_link = $row['share_link'];
        } else {
            $this->assignment_id = $assignment_id;
            $this->user_id = $user_id;
        }

        return $this;
    }

    public function getByPlagScanId(int $doc_id): self
    {
        global $DIC;

        $result = $DIC->database()->queryF(
            'SELECT * FROM psx_submissions WHERE plagscan_id = %s',
            [ilDBConstants::T_INTEGER],
            [$doc_id]
        );

        if ($row = $DIC->database()->fetchAssoc($result)) {
            $this->id = (int) $row['id'];
            $this->assignment_exc_obj = (int) $row['assignment_exc_obj'];
            $this->assignment_id = (int) $row['assignment_id'];
            $this->user_id = (int) $row['user_id'];
            $this->user_non_consent = $row['user_non_consent'] === '1';
            $this->submission_ilias_id = (int) $row['submission_ilias_id'];
            $this->plagscan_id = (int) $row['plagscan_id'];
            $this->plagscan_status = $row['plagscan_status'];
            $this->share_link = $row['share_link'];
        }

        return $this;
    }

    public function store(): void
    {
        global $DIC;
        $DIC->logger()->root()->info('PSX: Document / store called for ' . $this->plagscan_id . ' with status ' . $this->plagscan_status);
        if ($this->id !== 0) {
            $DIC->database()->manipulateF('DELETE FROM psx_submissions WHERE id = %s', [ilDBConstants::T_INTEGER], [$this->id]);
        }
        $this->id = $DIC->database()->nextId('psx_submissions');
        $DIC->database()->insert(
            'psx_submissions',
            [
                'id' => [ilDBConstants::T_INTEGER, $this->id],
                'assignment_exc_obj' => [ilDBConstants::T_INTEGER, $this->assignment_exc_obj],
                'assignment_id' => [ilDBConstants::T_INTEGER, $this->assignment_id],
                'user_id' => [ilDBConstants::T_INTEGER, $this->user_id],
                'user_non_consent' => [ilDBConstants::T_INTEGER, $this->user_non_consent ? '1' : '0'],
                'submission_ilias_id' => [ilDBConstants::T_INTEGER, $this->submission_ilias_id],
                'plagscan_id' => [ilDBConstants::T_INTEGER, $this->plagscan_id],
                'plagscan_status' => [ilDBConstants::T_INTEGER, $this->plagscan_status],
                'share_link' => [ilDBConstants::T_INTEGER, $this->share_link]
            ]
        );
    }
}
