<?php

/**
 * This file is part of ILIAS, a powerful learning management system
 * published by ILIAS open source e-Learning e.V.
 *
 * ILIAS is licensed with the GPL-3.0,
 * see https://www.gnu.org/licenses/gpl-3.0.en.html
 * You should have received a copy of said license along with the
 * source code, too.
 *
 * If this is not the case or you just want to try ILIAS, you'll find
 * us at:
 * https://www.ilias.de
 * https://github.com/ILIAS-eLearning
 *
 *********************************************************************/

declare(strict_types=1);

namespace ILIAS\Plugin\PlagScanExercise\Table;

use ilAssignmentsPerParticipantTableGUI;
use ilExAssignment;
use ilExAssignmentTeam;
use ilExSubmission;
use ilObjUser;

class ParticipantsTableGUI extends ilAssignmentsPerParticipantTableGUI
{
    protected ilObjUser $user;

    protected function parseData(): array
    {
        if (!isset($this->user)) {
            global $DIC;
            $this->user = $DIC->user();
        }

        if (
            is_array($info = $this->access->getInfo()) &&
            !$this->access->checkAccessOfUser($this->user->getId(), 'read', '', $this->exc->getRefId())) {
            $this->setDescription('<span class="warning">' . $info[0]['text'] . '</span>');
        }

        $data = [];
        foreach (ilExAssignment::getInstancesByExercise($this->exc->getId()) as $ass) {
            $member_status = $ass->getMemberStatus($this->user->getId());

            if ($this->filter['status'] && $member_status->getStatus() !== $this->filter['status']) {
                continue;
            }

            $submission = new ilExSubmission($ass, $this->user->getId());
            $idl = $ass->getIndividualDeadlines();

            if ($this->filter['subm']) {
                if ($this->filter['subm'] === 'y' && !$submission->getLastSubmission()) {
                    continue;
                }
                if ($this->filter['subm'] === 'n' && $submission->getLastSubmission()) {
                    continue;
                }
            }

            $row = [
                'ass' => $ass,
                'submission_obj' => $submission,
                'name' => $ass->getTitle(),
                'status' => $member_status->getStatus(),
                'mark' => $member_status->getMark(),
                'sent_time' => $member_status->getSentTime(),
                'status_time' => $member_status->getStatusTime(),
                'feedback_time' => $member_status->getFeedbackTime(),
                'submission' => $submission->getLastSubmission(),
                'notice' => $member_status->getNotice(),
                'comment' => $member_status->getComment(),
                'order_nr' => $ass->getOrderNr()
            ];

            if ($ass->hasTeam()) {
                $team_map = ilExAssignmentTeam::getAssignmentTeamMap($ass->getId());

                $row['team'] = [];
                if ($submission->getTeam() !== null) {
                    foreach ($submission->getTeam()->getMembers() as $user_id) {
                        $row['team'][$user_id] = ilObjUser::_lookupFullname($user_id);
                    }
                }
                asort($row['team']);

                $team_id = $team_map[$this->user->getId()];
                if (is_numeric($team_id)) {
                    $idl_team_id = 't' . $team_id;
                    if (array_key_exists($idl_team_id, $idl)) {
                        $row['idl'] = $idl[$idl_team_id];
                    }
                }
            } elseif (array_key_exists($this->user->getId(), $idl)) {
                $row['idl'] = $idl[$this->user->getId()];
            }

            $data[] = $row;
        }

        return $data;
    }

    protected function fillRow(array $set): void
    {
        $this->ctrl->setParameter($this->parent_obj, 'member_id', $this->user->getId());
        $this->ctrl->setParameter($this->parent_obj, 'ass_id', $set['ass']->getId());

        $this->tpl->setVariable('NAME_ID', 'ass');
        $this->tpl->setVariable('VAL_ID', $set['ass']->getId());

        $this->parseRow($this->user->getId(), $set['ass'], $set);

        $this->ctrl->setParameter($this->parent_obj, 'ass_id', '');
        $this->ctrl->setParameter($this->parent_obj, 'member_id', $this->user->getId());
    }
}
