<?php

/**
 * This file is part of ILIAS, a powerful learning management system
 * published by ILIAS open source e-Learning e.V.
 *
 * ILIAS is licensed with the GPL-3.0,
 * see https://www.gnu.org/licenses/gpl-3.0.en.html
 * You should have received a copy of said license along with the
 * source code, too.
 *
 * If this is not the case or you just want to try ILIAS, you'll find
 * us at:
 * https://www.ilias.de
 * https://github.com/ILIAS-eLearning
 *
 *********************************************************************/

declare(strict_types=1);

use ILIAS\Filesystem\Stream\Streams;
use ILIAS\Plugin\PlagScanExercise\Auth\FrontendCredentials;
use ILIAS\Plugin\PlagScanExercise\Auth\Provider;
use ILIAS\Plugin\PlagScanExercise\PlagScan\DocumentManagement;

global $DIC;
$logger = $DIC->logger()->root();
$http = $DIC->http();
$get = $http->request()->getQueryParams();
$server = $http->request()->getServerParams();
$cookie = $http->request()->getCookieParams();

$ilias_root_directory = './';
$cookie_path = dirname($server['PHP_SELF']);
$request_uri = $server['REQUEST_URI'];

for ($i = 0; !file_exists($ilias_root_directory . 'ilias.ini.php') && $i < 20; $i++) {
    $ilias_root_directory .= '../';
    $cookie_path = dirname($cookie_path);
    $request_uri = dirname($request_uri);
}

chdir($ilias_root_directory);

if (!file_exists(getcwd() . '/ilias.ini.php')) {
    die('Please ensure ILIAS is installed!');
}

$cookie_path .= (!preg_match("/[\/|\\\\]$/", $cookie_path)) ? "/" : '';

if ($cookie_path === "\\") {
    $cookie_path = '/';
}
if (isset($get['client_id'])) {
    setcookie('ilClientId', $get['client_id'], 0, $cookie_path, '');
    $cookie['ilClientId'] = $get['client_id'];
} else {
    setcookie('ilClientId', 'default', 0, $cookie_path, '');
    $cookie['ilClientId'] = 'default';
    $get['client_id'] = 'default';
}
define('IL_COOKIE_PATH', $cookie_path);

$server['REQUEST_URI'] = dirname($request_uri) . '/' . basename($server['REQUEST_URI']);
$server['PHP_SELF'] = str_replace('?' . $server['QUERY_STRING'], '', $server['REQUEST_URI']);

ilContext::init(ilContext::CONTEXT_RSS);
ilInitialisation::initILIAS();

$auth_status = ilAuthStatus::getInstance();
$auth_credentials = new FrontendCredentials(($get['token'] ?? ''));
$auth_provider = new Provider($auth_credentials);
$auth_frontend = new ilAuthFrontendWS($DIC['ilAuthSession'], $auth_status, $auth_credentials, [$auth_provider]);

$auth_frontend->authenticate();
$response = $http->response();
switch ($auth_status->getStatus()) {
    case ilAuthStatus::STATUS_AUTHENTICATED:
        $uri_segments = explode('?', $server['REQUEST_URI']);
        $logger->info('PSX: Endpoint call to ' . $server['REQUEST_URI']);
        if (is_numeric((int) $uri_segments[1]) && !isset($get['docID'])) {
            $logger->info('PSX: Endpoint call - Triggering update for docID ' . $uri_segments[1]);
            DocumentManagement::updateDocumentStatus((int) $uri_segments[1]);
        }
        $logger->info('PSX: Processing done, sending response, ending update for docID ' . $get['docID']);

        $response = $response
            ->withStatus(200)
            ->withHeader('Content-Type', 'application/json; charset=UTF-8;')
            ->withBody($stream = Streams::ofString(json_encode(['status' => 'authenticated'], JSON_THROW_ON_ERROR)));
        break;
    case ilAuthStatus::STATUS_AUTHENTICATION_FAILED:
        $response = $response->withStatus(401);
        break;
}
$http->saveResponse($response);
$http->sendResponse();
